/*
    This file is part of Escota.

    Escota is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Escota is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Escota.  If not, see <http://www.gnu.org/licenses/>.
 */
package escota;

import com.aetrion.flickr.groups.Group;
import escota.flickr.Conexion;
import escota.flickr.ExecuteTasksThread;
import escota.flickr.Rule;
import escota.flickr.RuleList;
import escota.flickr.Task;
import escota.flickr.TaskQueue;
import escota.flickr.UpdateDataThread;
import escota.utils.Log;
import org.jdesktop.application.Action;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import org.jdesktop.swingx.JXTable;

/**
 * The application's main frame.
 */
public class EscotaView extends FrameView {

    public EscotaView(SingleFrameApplication app) {
        super(app);

        initComponents();
        
        getFrame().setResizable(false);

        getFrame().setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        getFrame().addWindowListener(new WindowAdapter()
        {
            @Override
            public void windowClosing(WindowEvent we)
            {
                exit();
            }
        });

        try {
            conexion = new Conexion(this);
        } catch (Exception e) {
            
            Log.error("Error creating a new conexion", e);
            this.showErrorMessage("Error creating a new conexion",true);
        }
        showFlickrLogin();
        if(!conexion.canAuth())
            System.exit(0);
        new UpdateDataThread(conexion).start();
        showUpdatingData();
        
        if(!conexion.isDataUpdated())
            System.exit(0);
        ruleList = new RuleList(conexion.getNsid());
        ruleList.loadRuleListFromDisk();

        taskQueue = new TaskQueue(conexion.getNsid());
        taskQueue.loadTaskQueueFromDisk();
        jXTable1.setModel(taskQueue);
        //Populate groups to add photos
        refreshGroups();

        jXList1.setModel(ruleList);
        jXList1.updateUI();
        this.setEnableForm(false);
    }
    
    private void refreshGroups()
    {
        if(jComboBox1.getItemCount()!=0)
            jComboBox1.removeAllItems();
        Iterator<Group> git = conexion.getMyGroups().iterator();
        while(git.hasNext())
        {
            Group g =  git.next();
            jComboBox1.insertItemAt(g.getName(), jComboBox1.getItemCount());
        }

        if(jComboBox2.getItemCount()!=0)
            jComboBox2.removeAllItems();
        git = conexion.getMyGroups().iterator();
        while(git.hasNext())
        {
            Group g =  git.next();
            jComboBox2.insertItemAt(g.getName(), jComboBox2.getItemCount());
        }
    }
    @Action
    public void exit() {
        Log.debug("Closing application");
        File directory = new File(System.getProperty("user.home") + File.separatorChar + ".escota" + File.separatorChar + "photos");
        if (directory.exists())
            deleteDirectory(directory);
        try {

            if(ruleList!=null)
                ruleList.saveRuleListToDisk();
            if(taskQueue!=null)
                taskQueue.saveTaskQueueToDisk();
        } catch (Exception e) {
            String error = "Error saving the rule list and the task queue";
            
            Log.error(error, e);
            showErrorMessage(error,true);
        }
        System.exit(0);
    }

    private boolean deleteDirectory(File path) {
        if (path.exists()) {
            File[] files = path.listFiles();
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory()) {
                    deleteDirectory(files[i]);
                } else {
                    files[i].delete();
                }
            }
        }
        return (path.delete());
    }

    
    public void showErrorMessage(String error, boolean endApp) {
        JFrame mainFrame = EscotaApp.getApplication().getMainFrame();
        errorMessage = new EscotaErrorMessage(mainFrame, error, true, this, endApp);
        errorMessage.setLocationRelativeTo(mainFrame);
        EscotaApp.getApplication().show(errorMessage);
    }
    @Action
    public void showFlickrLogin() {
        if (flickrLogin == null) {
            JFrame mainFrame = EscotaApp.getApplication().getMainFrame();
            flickrLogin = new EscotaFlickrLogin(mainFrame, true, conexion,this);
            flickrLogin.setLocationRelativeTo(mainFrame);
        }
        EscotaApp.getApplication().show(flickrLogin);

    }
    @Action
    public void showPhotoChooser(Rule rule) {
        JFrame mainFrame = EscotaApp.getApplication().getMainFrame();
        photoChooser = new EscotaPhotoChooser(mainFrame, conexion, rule, taskQueue,this);
        photoChooser.setLocationRelativeTo(mainFrame);
        EscotaApp.getApplication().show(photoChooser);
    }
    @Action
    public void showUpdatingData() {
        if (updatingData == null) {
            JFrame mainFrame = EscotaApp.getApplication().getMainFrame();
            updatingData = new EscotaUpdatingData(mainFrame);
            updatingData.setLocationRelativeTo(mainFrame);
        }
        EscotaApp.getApplication().show(updatingData);
    }
    @Action
    public void showTaskExecutor() {
        JFrame mainFrame = EscotaApp.getApplication().getMainFrame();
        taskExecutor = new EscotaTaskExecutor(mainFrame, this);
        taskExecutor.setLocationRelativeTo(mainFrame);
        EscotaApp.getApplication().show(taskExecutor);
    }
    @Action
    public void showTaskGenerator() {
        JFrame mainFrame = EscotaApp.getApplication().getMainFrame();
        taskGenerator = new EscotaTaskGenerator(mainFrame);
        taskGenerator.setLocationRelativeTo(mainFrame);
        EscotaApp.getApplication().show(taskGenerator);
    }
    private void selectRule(int pos)
    {
        this.setEnableForm(true);

        Rule rule = (Rule) jXList1.getElementAt(pos);

        jTextField1.setText(rule.getName());
        jTextArea1.setText(rule.getDescription());
        jFormattedTextField1.setText(Integer.toString(rule.getMinViews()));
        jFormattedTextField2.setText(Integer.toString(rule.getMinFavs()));
        jFormattedTextField3.setText(Integer.toString(rule.getMinComments()));
        jFormattedTextField4.setText(Integer.toString(rule.getMaxViews()));
        jFormattedTextField5.setText(Integer.toString(rule.getMaxFavs()));
        if(rule.getTags()!=null)
            jTextField5.setText(formatTags(rule.getTags()));
        else
            jTextField5.setText("");
        if(rule.getDateTaken1()!=null)
            this.jXDatePicker1.setDate(rule.getDateTaken1());
        else
            this.jXDatePicker1.setDate(null);
        if(rule.getDateTaken2()!=null)
            this.jXDatePicker2.setDate(rule.getDateTaken2());
        else
            this.jXDatePicker2.setDate(null);
        if(rule.getDatePosted1()!=null)
            this.jXDatePicker3.setDate(rule.getDatePosted1());
        else
            this.jXDatePicker3.setDate(null);
        if(rule.getDatePosted2()!=null)
            this.jXDatePicker4.setDate(rule.getDatePosted2());
        else
            this.jXDatePicker4.setDate(null);

        selectGroupToAdd(rule.getGroupNameToAdd());
        selectGroupToDel(rule.getGroupNameToDel());

        this.lastSelectedRule = jXList1.getSelectedIndex();

    }
    private boolean savedRule(int pos)
    {
        Rule r = (Rule) ruleList.getElementAt(pos);
        r.setName(jTextField1.getText());
        r.setDescription(jTextArea1.getText());
        try
        {
            r.setMinViews(Integer.parseInt(jFormattedTextField1.getText()));
        }
        catch(NumberFormatException e)
        {
            jFormattedTextField1.setText("0");
            showErrorMessage("Minimum views must be a number",false);
            return false;
        }
        try
        {
            r.setMinFavs(Integer.parseInt(jFormattedTextField2.getText()));
        }
        catch(NumberFormatException e)
        {
            jFormattedTextField2.setText("0");
            showErrorMessage("Minimum favorites must be a number",false);
            return false;
        }
        try
        {
            r.setMinComments(Integer.parseInt(jFormattedTextField3.getText()));
        }
        catch(NumberFormatException e)
        {
            jFormattedTextField3.setText("0");
            showErrorMessage("Minimum views must be a number",false);
            return false;
        }
        try
        {
            r.setMaxViews(Integer.parseInt(jFormattedTextField4.getText()));
        }
        catch(NumberFormatException e)
        {
            jFormattedTextField4.setText("0");
            showErrorMessage("Maximum views must be a number",false);
            return false;
        }
        try
        {
            r.setMaxFavs(Integer.parseInt(jFormattedTextField5.getText()));
        }
        catch(NumberFormatException e)
        {
            jFormattedTextField5.setText("0");
            showErrorMessage("Maximum favorites must be a number",false);
            return false;
        }
        if(!jTextField5.getText().equals(""))
            this.saveTags(jTextField5.getText(), pos);
        else
            r.setTags(null);

        r.setDateTaken1(this.jXDatePicker1.getDate());
        r.setDateTaken2(this.jXDatePicker2.getDate());
        r.setDatePosted1(this.jXDatePicker3.getDate());
        r.setDatePosted2(this.jXDatePicker4.getDate());

        
        String groupNameToAdd = (String) jComboBox1.getSelectedItem();
        if(groupNameToAdd!=null)
        {
            String groupID = conexion.getGroupID(groupNameToAdd);
            r.setGroupToAdd(groupID, groupNameToAdd);
        }
        String groupNameToDel = (String) jComboBox2.getSelectedItem();
        if(groupNameToDel!=null)
        {
            String groupID = conexion.getGroupID(groupNameToDel);
            r.setGroupToDel(groupID, groupNameToDel);
        }

        return true;
    }
    private void selectGroupToAdd(String groupName)
    {
        jComboBox1.setSelectedIndex(-1);
        for(int i=0; i<jComboBox1.getItemCount();i++)
        {
            String item = (String) jComboBox1.getItemAt(i);
            if(item.equals(groupName))
            {
                jComboBox1.setSelectedIndex(i);
                break;
            }
        }
    }
    private void selectGroupToDel(String groupName)
    {
        jComboBox2.setSelectedIndex(-1);
        for(int i=0; i<jComboBox2.getItemCount();i++)
        {
            String item = (String) jComboBox2.getItemAt(i);
            if(item.equals(groupName))
            {
                jComboBox2.setSelectedIndex(i);
                break;
            }
        }
    }
    private String formatTags(List<String> tags)
    {
        Iterator<String> it = tags.iterator();
        String res = "";
        while(it.hasNext())
        {
            res += it.next();
            if(it.hasNext())
                res += " ";
        }
        return res;
    }
    private void saveTags(String tags, int pos)
    {
        String[]  result = tags.split("[ ]");
        List<String> temp = new ArrayList<String>();
        for(int i=0;i<result.length;i++)
        {
            temp.add(result[i]);
        }
        Rule r = (Rule) ruleList.getElementAt(pos);
        r.setTags(temp);
    }
    public void setEnableApp(Boolean enable)
    {
        if(!enable)
            savedRule(this.lastSelectedRule);
        this.jButton1.setEnabled(enable);
        this.jButton2.setEnabled(enable);
        this.jButton3.setEnabled(enable);
        this.jXList1.setEnabled(enable);
        this.jTabbedPane1.setEnabled(enable);
        this.jLabel1.setEnabled(enable);
        this.jLabel2.setEnabled(enable);
        this.jLabel3.setEnabled(enable);
        this.jLabel4.setEnabled(enable);
        this.jLabel5.setEnabled(enable);
        this.jLabel6.setEnabled(enable);
        this.jLabel7.setEnabled(enable);
        this.jLabel8.setEnabled(enable);
        this.jLabel9.setEnabled(enable);
        this.jLabel10.setEnabled(enable);
        this.jLabel11.setEnabled(enable);
        this.jTextField1.setEnabled(enable);
        this.jTextField5.setEnabled(enable);
        this.jTextArea1.setEnabled(enable);
        this.jFormattedTextField1.setEnabled(enable);
        this.jFormattedTextField2.setEnabled(enable);
        this.jFormattedTextField3.setEnabled(enable);
        this.jFormattedTextField4.setEnabled(enable);
        this.jFormattedTextField5.setEnabled(enable);
        this.jXDatePicker1.setEnabled(enable);
        this.jXDatePicker2.setEnabled(enable);
        this.jXDatePicker3.setEnabled(enable);
        this.jXDatePicker4.setEnabled(enable);
        this.jComboBox1.setEnabled(enable);
        this.jComboBox2.setEnabled(enable);
        this.jButton3.setEnabled(enable);
    }
    private void setEnableForm(Boolean enable)
    {
        this.jLabel1.setEnabled(enable);
        this.jLabel2.setEnabled(enable);
        this.jLabel3.setEnabled(enable);
        this.jLabel4.setEnabled(enable);
        this.jLabel5.setEnabled(enable);
        this.jLabel6.setEnabled(enable);
        this.jLabel7.setEnabled(enable);
        this.jLabel8.setEnabled(enable);
        this.jLabel9.setEnabled(enable);
        this.jLabel10.setEnabled(enable);
        this.jLabel11.setEnabled(enable);
        this.jTextField1.setEnabled(enable);
        this.jTextField5.setEnabled(enable);
        this.jTextArea1.setEnabled(enable);
        this.jFormattedTextField1.setEnabled(enable);
        this.jFormattedTextField2.setEnabled(enable);
        this.jFormattedTextField3.setEnabled(enable);
        this.jFormattedTextField4.setEnabled(enable);
        this.jFormattedTextField5.setEnabled(enable);
        this.jXDatePicker1.setEnabled(enable);
        this.jXDatePicker2.setEnabled(enable);
        this.jXDatePicker3.setEnabled(enable);
        this.jXDatePicker4.setEnabled(enable);
        this.jComboBox1.setEnabled(enable);
        this.jComboBox2.setEnabled(enable);
        this.jButton3.setEnabled(enable);
        if(!enable)
        {
            this.jTextField1.setText("");
            this.jTextField5.setText("");
            this.jTextArea1.setText("");
            this.jFormattedTextField1.setText("");
            this.jFormattedTextField2.setText("");
            this.jFormattedTextField3.setText("");
            this.jFormattedTextField4.setText("");
            this.jFormattedTextField5.setText("");
            this.jXDatePicker1.setDate(null);
            this.jXDatePicker2.setDate(null);
            this.jXDatePicker3.setDate(null);
            this.jXDatePicker4.setDate(null);
            this.jComboBox1.setSelectedIndex(-1);
            this.jComboBox2.setSelectedIndex(-1);
        }
    }

    public EscotaUpdatingData getUpdatingData() {
        return (EscotaUpdatingData) updatingData;
    }
    public JXTable getTaskTable()
    {
        return jXTable1;
    }
    public TaskQueue getTaskQueue()
    {
        return taskQueue;
    }
    public Conexion getConexion()
    {
        return conexion;
    }
    public EscotaTaskExecutor getTaskExecutor()
    {
        return (EscotaTaskExecutor) this.taskExecutor;
    }
    public EscotaTaskGenerator getTaskGenerator()
    {
        return (EscotaTaskGenerator) taskGenerator;
    }
    public void setNullTaskGenerator()
    {
        taskGenerator=null;
    }
    public EscotaPhotoChooser getPhotoChooser()
    {
        return (EscotaPhotoChooser) this.photoChooser;
    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jTextField5 = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        jFormattedTextField1 = new javax.swing.JFormattedTextField();
        jFormattedTextField2 = new javax.swing.JFormattedTextField();
        jFormattedTextField3 = new javax.swing.JFormattedTextField();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jComboBox1 = new javax.swing.JComboBox();
        jButton1 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jLabel11 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jXList1 = new org.jdesktop.swingx.JXList();
        jButton2 = new javax.swing.JButton();
        jXDatePicker1 = new org.jdesktop.swingx.JXDatePicker();
        jXDatePicker2 = new org.jdesktop.swingx.JXDatePicker();
        jXDatePicker3 = new org.jdesktop.swingx.JXDatePicker();
        jXDatePicker4 = new org.jdesktop.swingx.JXDatePicker();
        jLabel12 = new javax.swing.JLabel();
        jFormattedTextField4 = new javax.swing.JFormattedTextField();
        jLabel13 = new javax.swing.JLabel();
        jFormattedTextField5 = new javax.swing.JFormattedTextField();
        jLabel14 = new javax.swing.JLabel();
        jComboBox2 = new javax.swing.JComboBox();
        jPanel2 = new javax.swing.JPanel();
        jButton4 = new javax.swing.JButton();
        jButton6 = new javax.swing.JButton();
        jScrollPane3 = new javax.swing.JScrollPane();
        jXTable1 = new org.jdesktop.swingx.JXTable();

        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setPreferredSize(new java.awt.Dimension(760, 420));

        jTabbedPane1.setName("jTabbedPane1"); // NOI18N
        jTabbedPane1.setPreferredSize(new java.awt.Dimension(0, 0));

        jPanel1.setName("jPanel1"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(escota.EscotaApp.class).getContext().getResourceMap(EscotaView.class);
        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jTextField1.setText(resourceMap.getString("jTextField1.text")); // NOI18N
        jTextField1.setName("jTextField1"); // NOI18N

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        jTextArea1.setColumns(20);
        jTextArea1.setLineWrap(true);
        jTextArea1.setRows(3);
        jTextArea1.setName("jTextArea1"); // NOI18N
        jScrollPane2.setViewportView(jTextArea1);

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N

        jLabel5.setText(resourceMap.getString("jLabel5.text")); // NOI18N
        jLabel5.setName("jLabel5"); // NOI18N

        jLabel6.setText(resourceMap.getString("jLabel6.text")); // NOI18N
        jLabel6.setToolTipText(resourceMap.getString("jLabel6.toolTipText")); // NOI18N
        jLabel6.setName("jLabel6"); // NOI18N

        jTextField5.setText(resourceMap.getString("jTextField5.text")); // NOI18N
        jTextField5.setToolTipText(resourceMap.getString("jTextField5.toolTipText")); // NOI18N
        jTextField5.setName("jTextField5"); // NOI18N

        jLabel7.setText(resourceMap.getString("jLabel7.text")); // NOI18N
        jLabel7.setToolTipText(resourceMap.getString("jLabel7.toolTipText")); // NOI18N
        jLabel7.setName("jLabel7"); // NOI18N

        jFormattedTextField1.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
        jFormattedTextField1.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jFormattedTextField1.setText(resourceMap.getString("jFormattedTextField1.text")); // NOI18N
        jFormattedTextField1.setName("jFormattedTextField1"); // NOI18N

        jFormattedTextField2.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
        jFormattedTextField2.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jFormattedTextField2.setText(resourceMap.getString("jFormattedTextField2.text")); // NOI18N
        jFormattedTextField2.setName("jFormattedTextField2"); // NOI18N

        jFormattedTextField3.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
        jFormattedTextField3.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jFormattedTextField3.setText(resourceMap.getString("jFormattedTextField3.text")); // NOI18N
        jFormattedTextField3.setName("jFormattedTextField3"); // NOI18N

        jLabel8.setText(resourceMap.getString("jLabel8.text")); // NOI18N
        jLabel8.setName("jLabel8"); // NOI18N

        jLabel9.setText(resourceMap.getString("jLabel9.text")); // NOI18N
        jLabel9.setName("jLabel9"); // NOI18N

        jLabel10.setText(resourceMap.getString("jLabel10.text")); // NOI18N
        jLabel10.setName("jLabel10"); // NOI18N

        jComboBox1.setName("jComboBox1"); // NOI18N

        jButton1.setText(resourceMap.getString("jButton1.text")); // NOI18N
        jButton1.setName("jButton1"); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton3.setText(resourceMap.getString("jButton3.text")); // NOI18N
        jButton3.setName("jButton3"); // NOI18N
        jButton3.setPreferredSize(new java.awt.Dimension(0, 0));
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        jLabel11.setText(resourceMap.getString("jLabel11.text")); // NOI18N
        jLabel11.setName("jLabel11"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        jXList1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jXList1.setName("jXList1"); // NOI18N
        jXList1.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                jXList1ValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(jXList1);

        jButton2.setText(resourceMap.getString("jButton2.text")); // NOI18N
        jButton2.setName("jButton2"); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jXDatePicker1.setName("jXDatePicker1"); // NOI18N

        jXDatePicker2.setName("jXDatePicker2"); // NOI18N

        jXDatePicker3.setName("jXDatePicker3"); // NOI18N

        jXDatePicker4.setName("jXDatePicker4"); // NOI18N

        jLabel12.setText(resourceMap.getString("jLabel12.text")); // NOI18N
        jLabel12.setName("jLabel12"); // NOI18N

        jFormattedTextField4.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
        jFormattedTextField4.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jFormattedTextField4.setText(resourceMap.getString("jFormattedTextField4.text")); // NOI18N
        jFormattedTextField4.setName("jFormattedTextField4"); // NOI18N

        jLabel13.setText(resourceMap.getString("jLabel13.text")); // NOI18N
        jLabel13.setName("jLabel13"); // NOI18N

        jFormattedTextField5.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
        jFormattedTextField5.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jFormattedTextField5.setText(resourceMap.getString("jFormattedTextField5.text")); // NOI18N
        jFormattedTextField5.setName("jFormattedTextField5"); // NOI18N

        jLabel14.setText(resourceMap.getString("jLabel14.text")); // NOI18N
        jLabel14.setName("jLabel14"); // NOI18N

        jComboBox2.setName("jComboBox2"); // NOI18N

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 74, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 162, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel2)
                            .addComponent(jLabel3)
                            .addComponent(jLabel1)
                            .addComponent(jLabel12)
                            .addComponent(jLabel5, javax.swing.GroupLayout.PREFERRED_SIZE, 97, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel7)
                            .addComponent(jLabel14)
                            .addComponent(jLabel10)
                            .addComponent(jLabel8))
                        .addGap(37, 37, 37)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(jTextField1)
                            .addComponent(jComboBox2, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jComboBox1, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel1Layout.createSequentialGroup()
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addComponent(jFormattedTextField1)
                                    .addComponent(jFormattedTextField3)
                                    .addComponent(jXDatePicker1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(jXDatePicker2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(jFormattedTextField4, javax.swing.GroupLayout.PREFERRED_SIZE, 145, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(18, 18, 18)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel4)
                                    .addComponent(jLabel9)
                                    .addComponent(jLabel13)
                                    .addComponent(jLabel6)
                                    .addComponent(jLabel11))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addComponent(jXDatePicker4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(jTextField5)
                                    .addComponent(jXDatePicker3, javax.swing.GroupLayout.DEFAULT_SIZE, 146, Short.MAX_VALUE)
                                    .addComponent(jFormattedTextField5)
                                    .addComponent(jFormattedTextField2, javax.swing.GroupLayout.DEFAULT_SIZE, 120, Short.MAX_VALUE)))
                            .addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.LEADING)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(485, 485, 485)
                        .addComponent(jButton3, javax.swing.GroupLayout.DEFAULT_SIZE, 102, Short.MAX_VALUE)))
                .addGap(281, 281, 281))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel1)
                            .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel2)
                            .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(15, 15, 15)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel3)
                            .addComponent(jFormattedTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel4)
                            .addComponent(jFormattedTextField2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(18, 18, 18)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jFormattedTextField4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel12)
                            .addComponent(jLabel13)
                            .addComponent(jFormattedTextField5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(18, 18, 18)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel5)
                            .addComponent(jFormattedTextField3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel6)
                            .addComponent(jTextField5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(18, 18, 18)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel7)
                            .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jXDatePicker1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel9)
                                .addComponent(jXDatePicker3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                .addComponent(jXDatePicker2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(jLabel11)
                                    .addComponent(jXDatePicker4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addComponent(jLabel8))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel10)
                            .addComponent(jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel14)
                            .addComponent(jComboBox2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 326, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jButton1)
                        .addComponent(jButton2))
                    .addComponent(jButton3, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(25, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel1.TabConstraints.tabTitle"), jPanel1); // NOI18N

        jPanel2.setName("jPanel2"); // NOI18N

        jButton4.setText(resourceMap.getString("jButton4.text")); // NOI18N
        jButton4.setName("jButton4"); // NOI18N
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        jButton6.setText(resourceMap.getString("jButton6.text")); // NOI18N
        jButton6.setName("jButton6"); // NOI18N
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton6ActionPerformed(evt);
            }
        });

        jScrollPane3.setName("jScrollPane3"); // NOI18N

        jXTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null}
            },
            new String [] {
                "Photo to move", "Group to add photo", "Rule to apply", "Execute on", "Operation"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jXTable1.setName("jXTable1"); // NOI18N
        jScrollPane3.setViewportView(jXTable1);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 739, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                        .addComponent(jButton4)
                        .addGap(18, 18, 18)
                        .addComponent(jButton6)))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 328, Short.MAX_VALUE)
                .addGap(18, 18, 18)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton6)
                    .addComponent(jButton4))
                .addContainerGap())
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel2.TabConstraints.tabTitle"), jPanel2); // NOI18N

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 764, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 419, Short.MAX_VALUE)
        );

        setComponent(mainPanel);
    }// </editor-fold>//GEN-END:initComponents

    private void jXList1ValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_jXList1ValueChanged
        if(this.lastSelectedRule!=-1)
        {
            if(savedRule(this.lastSelectedRule))
            {
                selectRule(jXList1.getSelectedIndex());
            }
            else
            {
                jXList1.setSelectedIndex(lastSelectedRule);
            }
        }
        else
        {
            selectRule(jXList1.getSelectedIndex());
        }

        jXList1.updateUI();

    }//GEN-LAST:event_jXList1ValueChanged

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        String newRuleName = "Rule";

        int i = 1;
        while(ruleList.existRule(newRuleName+Integer.toString(i)))
        {
            i++;
        }
        newRuleName += Integer.toString(i);

        Rule newRule = new Rule(newRuleName);
        ruleList.add(newRule);
        jXList1.updateUI();
        if(ruleList.getSize()==1)
        {
            selectRule(0);
        }
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        if(jXList1.getSelectedIndex()!=-1)
        {
            int posToDelete = jXList1.getSelectedIndex();
            if(posToDelete == jXList1.getElementCount()-1)
            {
                jXList1.setSelectedIndex(jXList1.getSelectedIndex()-1);
                ruleList.remove(posToDelete);
                jXList1.updateUI();
            }
            else
            {
                ruleList.remove(posToDelete);
                jXList1.updateUI();
                selectRule(jXList1.getSelectedIndex());
            }
            if(jXList1.getElementCount()==0)
                this.setEnableForm(false);
        }
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed

        if((jComboBox1.getSelectedIndex()==-1)&&(jComboBox2.getSelectedIndex()==-1))
            this.showErrorMessage("Please, select a group", false);
        else if(jXList1.getSelectedIndex()==-1)
            this.showErrorMessage("Please, select a rule to apply", false);
        else
        {
            if(savedRule(jXList1.getSelectedIndex()))
            {
                Rule rule = ruleList.get(jXList1.getSelectedIndex());
                this.showPhotoChooser(rule);
                photoChooser = null;
                jTabbedPane1.setSelectedIndex(1);
                jXTable1.updateUI();
            }
        }
    }//GEN-LAST:event_jButton3ActionPerformed

    private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton6ActionPerformed
        new ExecuteTasksThread(this).start();
        this.showTaskExecutor();
        jXTable1.updateUI();
        this.taskExecutor=null;
    }//GEN-LAST:event_jButton6ActionPerformed

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        int pos = jXTable1.getSelectedRow();
        int i=0;
        Iterator<Task> it = taskQueue.iterator();
        while(it.hasNext())
        {
            it.next();
            if(i==pos)
            {
                it.remove();
                break;
            }
            i++;
        }
        jXTable1.updateUI();
    }//GEN-LAST:event_jButton4ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton6;
    private javax.swing.JComboBox jComboBox1;
    private javax.swing.JComboBox jComboBox2;
    private javax.swing.JFormattedTextField jFormattedTextField1;
    private javax.swing.JFormattedTextField jFormattedTextField2;
    private javax.swing.JFormattedTextField jFormattedTextField3;
    private javax.swing.JFormattedTextField jFormattedTextField4;
    private javax.swing.JFormattedTextField jFormattedTextField5;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField5;
    private org.jdesktop.swingx.JXDatePicker jXDatePicker1;
    private org.jdesktop.swingx.JXDatePicker jXDatePicker2;
    private org.jdesktop.swingx.JXDatePicker jXDatePicker3;
    private org.jdesktop.swingx.JXDatePicker jXDatePicker4;
    private org.jdesktop.swingx.JXList jXList1;
    private org.jdesktop.swingx.JXTable jXTable1;
    private javax.swing.JPanel mainPanel;
    // End of variables declaration//GEN-END:variables

    /*private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;*/

    private JDialog errorMessage;
    private JDialog flickrLogin;
    private JDialog photoChooser;
    private JDialog updatingData;
    private JDialog taskExecutor;
    private JDialog taskGenerator;
    private Conexion conexion;
    private RuleList ruleList;
    private TaskQueue taskQueue;
    private int lastSelectedRule=-1;
}
